import com.jinsight.jetchart.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;

/**
 * A target  zone is a  color  filled polygon  horizontally  or  vertically  painted  across
 * the chart area.  The goal  of  a target  zone is to  set upper and lower  limits  between
 * which  a  sequence of  data points  can  be  plotted  and   compared  against   delimiter
 * values. In this  application  we  can  see the  heartbeat  monitoring results of a person 
 * who  worked out for 30 minutes.  The  results  are  plotted  against  five  ranges,  each
 * one representing a target zone where different fitness levels can be achieved.
 * Graph data is read from an xml file, which is located in  the same directory  where  this 
 * application  can  be found. This is a Swing application, hence it requires that the Swing
 * compatible 'sGraphChart.jar' file be included in the  classpath.
 * The 'sGraphChart.jar' file can be found in the JetChart/lib directory after decompression
 * of the JetChart package.
 * @author Andre de Lima Soares - alsoares@jinsight.com
 */
public class Main extends JFrame {
    
    Color[] colors;
    
    String[] texts;
    
    Font targetLineFont;
    
    Graph graph;
    
    public Main() {
	
        setTitle("The JetChart Library - Target zone demo");

	// Adds a window adapter to close application.
	addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent evt) {
		    System.exit(0);
		}
	    });
	
	// Target zones and target lines are painted in these
	// colors.
	colors=new Color[]{new Color(255,255,153),
			   new Color(102,255,153),
			   new Color(51,204,255),
			   new Color(204,102,204),
			   new Color(255,102,102)};
	
	// Target lines texts.
	texts=new String[]{"Recovery (115-130 bpm)",
			   "Endurance (130-145 bpm)",
			   "Aerobic (145-165 bpm)",
			   "LT Training (165-180 bpm)",
			   "Anaerobic (180-200 bpm)"};
	
	// Target lines font.
	targetLineFont=new Font("Dialog",Font.PLAIN,10);
	
	createChartContext();
	
	createTargetLines();
	
	createTargetZones();
	
	setScaleProperties();
	
	createLineSeries();
	
	// Sets the left title text.
	LeftTitle lt=graph.getLeftTitle();
	lt.setText("Heartbeat");
	
	GraphSet graphSet=graph.getGraphSet(0);
	
	// Disables the horizontal axis and sets the vertical
	// axis color to gray.
	graphSet.getXAxis().setEnabled(false);
	graphSet.getYAxis().setColor(Color.gray);
	
	// Disables the primary GraphSet object labels and ticks.
	graphSet.setLabelsEnabled(false);
	graphSet.setLabelsMarksEnabled(false);
	
	// Enables tooltip. Multiple  tooltips  display is enabled to
	// provide tooltip reliance on the horizontal coordinates only.
	ToolTip tt=graph.getToolTip();
	tt.setEnabled(true);
	tt.setMultipleEnabled(true);
	
	// Enables the scrolling label. The scrolling label is displayed above
	// the chart area while mouse cursor is moved across the chart area.
	ScrollingLabel sl=graph.getScrollingLabel();
	sl.setEnabled(true);
	
	// Enables the scrolling bar. The scrolling bar cuts the line series at
	// the data point associated with a tooltip that is simultaneously displayed.
	ScrollingBar sb=graphSet.getScrollingBar();
	sb.setEnabled(true);
	
	// Enables the grid vertical lines. The only vertical grid line  we want
	// displayed is the first from left to right, to separate the target lines
	// text area from the chart area.
	GridCrossedLines gc=graphSet.getGridCrossedLines();
	gc.setEnabled(true);
	gc.setVisibleLinesIndexes(new int[]{0});
	
	// Enables a wall behind the chart area. The wall shadow is also enabled,
	// painted in gray.
	Wall wall=graphSet.getWall();
	wall.setEnabled(true);
	wall.setShadowEnabled(true);
	wall.setShadowColor(Color.gray);
	
	// Reads data from the xml file.
	try {
	    FileReader fr=new FileReader("chart.xml");
	    graph.readXMLData(fr);
	}
	catch (IOException e) {
	    e.printStackTrace();
	}
	
	Container ct=getContentPane();
	
	ct.add("Center",graph);
	
	setSize(600,400);
	
	setVisible(true);
	
    }
    
    private void createChartContext() {
	
	graph=new Graph();
	graph.setTitle(new String[]{"Heartbeat Monitoring","30 minutes workout performance graph"});
	graph.setTitleFont(new Font("Dialog",Font.BOLD,14));
	
	// Loads a gif file to be displayed on the background.
	Image img=Toolkit.getDefaultToolkit().getImage("workout.gif");
	graph.setBackgroundImage(img,Graph.TOP_LEFT);
	
	// Sets a start gap wide enough to fit the target line texts.
	FontMetrics fm=Toolkit.getDefaultToolkit().getFontMetrics(targetLineFont);
	int width=0;
	// Calculates the width of the largest text.
	for (int counter=0;counter<texts.length;counter++) 
	    width=Math.max(width,fm.stringWidth(texts[counter]));
	// Sets the start gap, including 10 pixels as a safety margin.
	int startGap=width+10;
	graph.setLabelsAxisStartGap(startGap);
	
	// Both start and end gaps are 10 pixels by default. As we want the target zones
	// ending at the rightmost coordinate of the horizontal axis, we have to set the
	// end gap to 0.
	graph.setLabelsAxisEndGap(0);
	
	// The method below sets the format of values displayed in tooltips. Decimal 
	// places are ignored.
	graph.setValueFormat("###");
	
    }
    
    private void createTargetLines() {
	
	// The static method below causes all TargetLine objects to be
	// painted behind the grid and all series. By default, target
	// lines are always painted in front of the grid and all series.
	TargetLine.setPosition(TargetLine.BEHIND);
	
	// All target lines values.
	double[] values={115,130,145,165,180};
	
	// Target lines are objects held by a GraphSet instance.
	GraphSet graphSet=graph.getGraphSet(0);
	
	// Starts a loop to create 5 target lines.
	for (int counter=0;counter<5;counter++) {
	    TargetLine tl=new TargetLine(values[counter]);
	    tl.setColor(colors[counter]);
	    tl.setText(texts[counter]);
	    tl.setTextPosition(TargetLine.ABOVE_LEFT);
	    tl.setFont(targetLineFont);
	    graphSet.addTargetLine(tl);
	}
	
    }
    
    private void createTargetZones() {
	
	// The static method below causes all Targetzone objects to be
	// painted behind the grid. by default, target zones are always
	// painted in front of the grid.
	TargetZone.setPosition(TargetZone.BEHIND);
	
	// All target zones values.
	double[][] lowerValues={{115},{130},{145},{165},{180}};
	double[][] upperValues={{130},{145},{165},{180},{200}};
	
	// Target zones are objects held by a GraphSet instance.
	GraphSet graphSet=graph.getGraphSet(0);
	
	// Starts a loop to create 5 TargetZone objects.
	for (int counter=0;counter<5;counter++) {
	    TargetZone tz=new TargetZone();
	    tz.setValues(lowerValues[counter],upperValues[counter]);
	    
	    // If the method setStretchEnabled is set to true, the target
	    // zones are stretched to fit the entire chart area.
	    tz.setStretchEnabled(true);
	    tz.setColor(colors[counter]);
	    tz.setBorderEnabled(false);
	    graphSet.addTargetZone(tz);
	}
    }
    
    
    private void setScaleProperties() {
	
	// Scales are objects held by a GraphSet instance.
	GraphSet graphSet=graph.getGraphSet(0);
	
	Scale scale=graphSet.getScale();
	
	// Disables automatic scale and sets the maximum, minimum and increment values.
	scale.setAutoScaleEnabled(false);
	scale.setMaxValue(200);
	scale.setMinValue(90);
	scale.setIncrement(20);
	
	// Sets scale values format. Decimal places are not displayed.
	scale.setValueFormat("###");
    }
    
    private void createLineSeries() {
	
	LineSerie ls=new LineSerie();
	ls.setTitle("Heartbeat");
	
	// Sets the series id to 0. The series id is checked by the xml parser to identify
	// a series to assign values read from an xml data source.
	ls.setId(0);
	ls.setColor(Color.black);
	ls.setMarksEnabled(false);
	
	graph.addSerie(ls);
	
    }
    
    public static void main(String[] args) {
	new Main();
    }
}


